@@ -25,7 +25,7 @@
 
 #include "memdebug.h"
 
-int test(char *URL)
+CURLcode test(char *URL)
 {
    CURLcode res;
    CURL *curl;
@@ -55,18 +55,11 @@ int test(char *URL)
 
    /*
     * Change the FTP_FILEMETHOD option to use full paths rather than a CWD
-    * command.  Alter the URL's path a bit, appending a "./".  Use an innocuous
-    * QUOTE command, after which curl will CWD to ftp_conn->entrypath and then
-    * (on the next call to ftp_statemach_act) find a non-zero ftpconn->dirdepth
-    * even though no directories are stored in the ftpconn->dirs array (after a
-    * call to freedirs).
+    * command. Use an innocuous QUOTE command, after which curl will CWD to
+    * ftp_conn->entrypath and then (on the next call to ftp_statemach_act)
+    * find a non-zero ftpconn->dirdepth even though no directories are stored
+    * in the ftpconn->dirs array (after a call to freedirs).
     */
-   newURL = aprintf("%s./", URL);
-   if(!newURL) {
-     curl_easy_cleanup(curl);
-     curl_global_cleanup();
-     return TEST_ERR_MAJOR_BAD;
-   }
 
    slist = curl_slist_append(NULL, "SYST");
    if(!slist) {
@@ -76,7 +69,7 @@ int test(char *URL)
      return TEST_ERR_MAJOR_BAD;
    }
 
-   test_setopt(curl, CURLOPT_URL, newURL);
+   test_setopt(curl, CURLOPT_URL, libtest_arg2);
    test_setopt(curl, CURLOPT_FTP_FILEMETHOD, (long) CURLFTPMETHOD_NOCWD);
    test_setopt(curl, CURLOPT_QUOTE, slist);
 
@@ -89,5 +82,5 @@ test_cleanup:
    curl_easy_cleanup(curl);
    curl_global_cleanup();
 
-   return (int)res;
+   return res;
 }
